import express from 'express';
import { Server } from 'socket.io';
import http from 'http';
import cors from 'cors';
import UserRepository from "./repositories/User";
import UserService from "./services/User";
import MapRepository from "./repositories/Map";
import prisma from "./helpers/prisma";
import { registerUser, loginUser } from './helpers/http';

const app = express();
const server = http.createServer(app);
const io = new Server(server, { cors: { origin: '*' } });

app.use(cors());
app.use(express.json());

app.post('/login', loginUser);
app.post('/register', registerUser);

async function handleSocketConnection(socket: any) {
    const {username, password} = socket.handshake.query;

    if (!username || !password) {
        socket.disconnect(true);
        return;
    }

    const user = UserRepository.getByUsername(username);

    if (!user || !await UserService.validateUserCredentials(username, password)) {
        socket.disconnect(true);
        return;
    }

    console.log('User connected:', username);

    socket.user = user;

    socket.emit('message', 'Welcome to the server!');

    const map = await MapRepository.getFirst();
    socket.join(map.name);
    socket.emit('message', 'You have joined the room: ' + map.name);

    socket.on('get_map', () => {
        console.log('Sending map to user:', username);
        socket.emit('map', map);
    });

    socket.on('move', (coords: any) => {
        // const player = map.players.find((p: any) => p.username === socket.user.username);
        // if (!player) return;
        // player.coords = coords;
        // io.to(map.name).emit('player_moved', map);
    });

    socket.on('disconnect', () => {
        console.log('User disconnected:', username);
    });
}

io.on('connection', handleSocketConnection);

const PORT = process.env.PORT || 3000;
server.listen(PORT, () => console.log(`Server running on port ${PORT}`));
